#!/bin/sh

#------------------------------------------------------------------------------
# auxiliary functions for epages shell scripts
# -----------------------------------------------------------------------------

# operating system

umask 027
NOHUP=nohup
AWK=awk

export NOHUP AWK
unset LANG LC_ALL LC_CTYPE LC__FASTMSG I_WANT_A_BROKEN_PS

unset EPAGES_KERNEL
UNAME_R=$(uname -r)
cnt=0
for i in ${UNAME_R//[-.]/ } ; do
  EPAGES_KERNEL=$EPAGES_KERNEL.$i
  cnt=$(($cnt + 1))
  [[ $cnt -ge 3 ]] && break
done

export EPAGES_KERNEL=${EPAGES_KERNEL#.}

if [ -f /etc/redhat-release ] ; then
  EPAGES_KERNEL=${EPAGES_KERNEL}r
elif [ -f /etc/SuSE-release ] ; then
  [[ $EPAGES_KERNEL != ${EPAGES_KERNEL#3.0.} ]] && EPAGES_KERNEL=2.6.32
  EPAGES_KERNEL=${EPAGES_KERNEL}s
fi

[[ $(uname -m) = x86_64 ]] && EPAGES_ARCH=x86_64 || EPAGES_ARCH=i386
export EPAGES_ARCH

# redhat 5 uses libxslt.so.1 / libexslt.so.0 / libxml2.so.2
# compiled by epages stored in $EPAGES_PERL/lib32/2.6.18r

if [[ -w /etc/passwd && $EPAGES_KERNEL = 2.6.18r ]] ; then
  for i in libxslt.so.1 libexslt.so.0 libxml2.so.2 ; do
    for j in $(ls -d /usr/lib/$i{,.*} 2>/dev/null) ; do
      [[ -f $j ]] || continue
      [[ -e /lib/${j##*/} ]] || cp -pP $j /lib
    done
    [[ -f /usr/lib/$i && -f "$(readlink -qf /lib/$i)" ]] && rm -f /usr/lib/$i
  done
fi

# set start column COL for echo

export column
column() {
    [[ -z $TERM || $TERM = raw || $TERM = dumb ]] && { echo -n "    "; return; }
    local COL=$1
    echo -en "\\033[${COL:-0}G"
}

# em(phasize)

export em R G B Y P C
em() {
  local COLOR BGCOLOR OPTIONS
  unset COLOR BGCOLOR OPTIONS
  if [[ $# -le 1 ]] ; then
    OPTIONS='-s(trong) -d(im) -u(nderline) -i(nverse) -h(idden) -x(out)'
    COLOR='-L(black) -R(ed) -G(reen) -B(lue) -Y(ellow) -P(ink) -C(yan) -W(hite)'
    BGCOLOR='-l(black) -r(ed) -g(reen) -b(lue) -y(ellow) -p(ink) -c(yan) -w(hite)'
    echo 'em [OPTIONS] [COLOR] [BGCOLOR] STRING'
    echo "OPTIONS: $OPTIONS"
    echo "COLOR:   $COLOR"
    echo "BGCOLOR: $BGCOLOR"
    return 1
  fi

  COLOR=39 # default
  BGCOLOR=49 # default
  local OPTIND=0
  while getopts sduihxLRGBYPCWlrgbypcw c $@ ; do
    case $c in
      s) OPTIONS="$OPTIONS\\033[1m" ;; # strong
      d) OPTIONS="$OPTIONS\\033[2m" ;; # dim
      u) OPTIONS="$OPTIONS\\033[4m" ;; # underline
      i) OPTIONS="$OPTIONS\\033[7m" ;; # inverse
      h) OPTIONS="$OPTIONS\\033[8m" ;; # hidden
      x) OPTIONS="$OPTIONS\\033[9m" ;; # crossout
      L) COLOR=30 ;; # black
      R) COLOR=31 ;; # red
      G) COLOR=32 ;; # green
      B) COLOR=34 ;; # blue
      Y) COLOR=33 ;; # yellow
      P) COLOR=35 ;; # pink
      C) COLOR=36 ;; # cyan
      W) COLOR=37 ;; # white
      l) BGCOLOR=40 ;; # black
      r) BGCOLOR=41 ;; # red
      g) BGCOLOR=42 ;; # green
      b) BGCOLOR=44 ;; # blue
      y) BGCOLOR=43 ;; # yellow
      p) BGCOLOR=45 ;; # pink
      c) BGCOLOR=46 ;; # cyan
      w) BGCOLOR=47 ;; # white
    esac
  done
  shift $(($OPTIND - 1))

  [[ -z $TERM || $TERM = raw || $TERM = dumb ]] && { echo "$@"; return; }
  echo -en "$OPTIONS\\033[${BGCOLOR};${COLOR}m$@\\033[m"
}

R() { em -R $@ ; }
G() { em -G $@ ; }
B() { em -B $@ ; }
Y() { em -Y $@ ; }
P() { em -P $@ ; }
C() { em -C $@ ; }

# timeout_test

export timeout_test
timeout_test() {
  if [[ $# -le 1 ]] ; then
    echo 'timeout_test FILE_TEST_OPERATOR TEST_OBJECT'
    echo "FILE_TEST_OPERATOR: see 'man test', e.g. one of: -f -d -e -x"
    echo "TEST_OBJECT: file/directory to test"
    return 1
  fi
  local HOW=$1 ; shift
  local TEST=/bin/test
  [ -x /usr/bin/test ] && TEST=/usr/bin/test
  /var/epages/exec.sh -c "$TEST $HOW '$@'"
  return $?
}

[[ $EPAGES_TEST = timeout_test ]] && alias test=timeout_test

# pathmunge

export pathmunge
pathmunge() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'pathmunge [SOURCE_PATH] TARGET_PATH'
    echo "TARGET_PATH: semicolon-separated path to change"
    echo "SOURCE_PATH: semicolon-separated path to prepend"
    echo '$(pathmunge /usr/bin:/bin:/usr/bin) == /usr/bin:/bin'
    echo '$(pathmunge /usr/bin /bin:/usr/bin) == /bin:/usr/bin'
    echo '$(pathmunge /usr/bin /bin)          == /usr/bin:/bin'
    return 1
  fi
  [[ $# -eq 0 ]] && return

  local IN=$2
  local ALL=${IN:+:$IN:}
  local OUT
  local OIFS=$IFS
  IFS=:
  for i in $1 ; do
    [ -z "$i" ] && continue
    test -d "$i" || continue
    [ "${ALL%:$i:*}" = "${ALL#*:$i:}" ] || continue
    ALL="$ALL:$i"
    OUT="$OUT:$i"
  done
  IFS=$OIFS
  [[ -n $IN && -z $OUT ]] && ALL=$IN
  [[ -n $IN && -n $OUT ]] && ALL="${OUT#:}:$IN"
  echo "${ALL#:}"
}

# ipcrm_user

export ipcrm_user
ipcrm_user() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'ipcrm_user USER'
    echo "USER: run ipcrm for UNIX user"
    return 1
  fi
  local user=$1
  [ -z "`ps -u$user -opid= 2>/dev/null`" ] || return
  ipcs -s >/dev/null 2>&1  || return

  ipcs -s | grep $user | while read T ID ELSE ; do ipcrm -s $ID ; done
  ipcs -m | grep $user | while read T ID ELSE ; do ipcrm -m $ID ; done
}

# get pid from /proc/

export checkpid
checkpid() {
    local i
    for i in $* ; do
        [ -d "/proc/$i" ] && return 0
    done
    return 1
}

# get pid from pidof

export get_pidof
get_pidof() {
    local c=-c
    [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%s} ]] || c=
    pidof $c -o $$ -o $PPID -o %PPID -x "$1" || \
        pidof $c -o $$ -o $PPID -o %PPID -x "${1##*/}"
}

# get pid from /var/run pid file

export get_pidfile
get_pidfile() {
    # check syntax
    if [ "$#" = 0 ]; then
        echo "Usage: get_pidfile [-p pidfile] [/path/to/program]"
        return 1
    fi

    local PID_FILE=
    [ "$1" = "-p" ] && { PID_FILE=$2; shift 2; } || PID_FILE=/var/run/${1##*/}.pid
    PID_FILE=${PID_FILE:-/var/run/${1##*/}.pid}

    local PID=
    if [ -n "$1" ] ; then
      local ARGS=$(echo $1 | tr -d '"' | tr -d "'" | sed 's/  */ /g;s/[]\(|){}^$*+?[]/./g')
      PID=$(pgrep -f "$ARGS")
      if [ -n "$PID" ] ; then
        PID=$( (cat "$PID_FILE" 2>/dev/null ; echo "$PID") | sort | uniq )
        sh -c "echo \"$PID\" > \"$PID_FILE\"" 2>/dev/null
      fi
    fi

    [ -s "$PID_FILE" ] || return 3 # program is not running
    [ -r "$PID_FILE" ] || return 4 # user has insufficient privilege

    PID=
    while read LINE ; do
        [ -z "$LINE" ] && break
        for i in $LINE ; do
            [ -z "${p//[0-9]/}" -a -d "/proc/$i" ] && PID="$PID $i"
        done
    done < "$PID_FILE"

    [ -n "${PID# }" ] && { echo ${PID# }; return 0; }
    return 1 # program is dead and /var/run pid file exists
}

# get_pids

export COLUMNS=1024

export get_pids
get_pids() {
  local PARENTS CHILDREN COMMAND ACCOUNT PIDS PPIDS CPIDS PROCESS
  unset PARENTS CHILDREN COMMAND ACCOUNT PIDS PPIDS CPIDS PROCESS
  local OPTIND=0
  while getopts PCc:u:p:h? c $@ ; do
    case $c in
      P) PARENTS=1 ;;
      C) CHILDREN=1 ;;
      c) COMMAND=$OPTARG ;;
      u) ACCOUNT=$OPTARG ;;
      p) PROCESS=$OPTARG ;;
      [h?]) true ;;
    esac
  done

  if [[ -z "$COMMAND$PROCESS" ]] ; then
    echo 'get_pids {-c COMMAND | -p PID} [-u USER] [-C] [-P]'
    echo "-c COMMAND: returns space separated PID list for command"
    echo "-p PID: returns space separated PID list for PID"
    echo "-u USER: returns only processes of USER"
    echo "-C: add child processes to PID list"
    echo "-P: add parent process to PID list"
    return 1
  fi

  if [[ -n $PROCESS ]] ; then
    PIDS=$(ps -p${PROCESS// /,} -opid= | sort | uniq | tr '\n' ',' | tr -d ' ')
  elif [[ -n $ACCOUNT ]] ; then
    PIDS=$(pgrep -u "$ACCOUNT" -f "$COMMAND" | tr '\n' ',')
  else
    PIDS=$(pgrep -f "$COMMAND" | tr '\n' ',')
  fi
  [[ -z ${PIDS%,} ]] && return

  if [[ -n $PARENTS ]] ; then
    PPIDS=$(ps -p${PIDS%,} -oppid= | sort | uniq | tr '\n' ',' | tr -d ' ')
  fi

  [[ -n $CHILDREN ]] && CPIDS=$(pgrep -P${PIDS%,} | tr '\n' ',')

  PIDS="$PPIDS$PIDS$CPIDS"
  echo ${PIDS//,/ }
}

# show_pids

export show_pids
show_pids() {
  if [[ $1 != ${1#-} ]] ; then
    echo '[PS_FORMAT=...] show_pids PID_LIST'
    echo "PID_LIST: show processes of space separated list in PS_FORMAT"
    return 1
  fi
  [[ $# -eq 0 ]] && return
  local PIDS=$@
  PS_FORMAT=${PS_FORMAT:-user,pid,ppid,nice,pri,pcpu,time,pmem,vsz,etime,args}
  [[ $# -gt 0 ]] && ps -p${PIDS// /,} -o$PS_FORMAT
}

# show_daemon

export show_daemon
show_daemon() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'show_daemon DAEMON'
    echo "DAEMON: show processes of daemon in PS_FORMAT"
    return 1
  fi
  local DAEMON=$1 ; shift
  [[ -z "$DAEMON" ]] && return

  unset PIDS
  case "$EPAGES_KERNEL" in
    # redhat
    *r) LANG=C /etc/init.d/$DAEMON status >/tmp/daemon.~$$~ \
        && PIDS=$(awk '/pid/{gsub(/[^0-9]/,"",$3); print $3}' /tmp/daemon.~$$~) ;;
    # suse
    *s) for pidfile in $(grep -i '/var/.*\.pid' /etc/init.d/$DAEMON  | sed -r 's,.*(/var/(\w|[+,./:=@~-])*).*,\1,' | grep -i '\.pid$') ; do
          [[ -s "$pidfile" ]] || continue
          PIDS=$(cat "$pidfile" | tr -cd '0-9')
          [[ -n $PIDS ]] && break
        done ;;
  esac

  [[ -n ${PIDS//[^0-9]/} ]] && PIDS=$(get_pids -p "$PIDS" $@)
  [[ -n ${PIDS//[^0-9]/} ]] && show_pids "$PIDS" || cat /tmp/daemon.~$$~
  rm -f /tmp/daemon.~$$~
}

# softkill

export softkill
softkill() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'softkill PID_LIST'
    echo "PID_LIST: space separated list of processes to kill; first try TERM, then KILL"
    return 1
  fi
  local PID_LIST=$@
  [[ $# -eq 0 ]] && return

  kill -15 $PID_LIST 2>/dev/null
  sleep 2

  PID_LIST=$(get_pids -p "$PID_LIST" -C)
  [[ -z $PID_LIST ]] && { echo SIGTERM; return; }

  kill -9 $PID_LIST 2>/dev/null
  sleep 2
  echo SIGKILL
}

# show result of runscript

export rs_show
rs_show() {
    local RC=${1:-$?}
    if [[ $CONSOLE != /dev/console ]] ; then
      column 60
      [ $RC = 0 ] && RET="  $(G OK)  " || RET="$(R FAILED)"
      echo -e "[$RET]\r"
    fi
    return $RC
}

# show that runscript is uninvolved

export rs_uninvolved
rs_uninvolved() {
    if [[ $CONSOLE != /dev/console ]] ; then
      echo -n "Starting $1 daemon: "
      column 56 
      echo -e "[$(em -Y -d UNINVOLVED)]\r"
    fi
}

# show status of runscript

export rs_status
rs_status() {
    # check syntax
    if [ "$#" = 0 ] ; then
        echo "Usage: rs_status [-p pidfile] [-l lockfile] [program]"
        return 1
    fi

    local PID PID_FILE LOCK_FILE
    unset PID PID_FILE LOCK_FILE
    while [ $# -gt 0 ] ; do
        case $1 in
            -p) PID_FILE=$2 ; shift ; shift ;;
            -l) LOCK_FILE=$2 ; shift ; shift ;;
             *) break ;;
        esac
    done

    if [[ ( -n ${!PID_FILE*} && -z $PID_FILE ) ||
          ( -n ${!LOCK_FILE*} && -z $LOCK_FILE ) ]] ; then
        echo "Usage: rs_status [-p pidfile] [-l lockfile] [program]"
        return 1
    fi

    if [[ -z $LOCK_FILE ]] ; then
        LOCK_FILE=${PID_FILE##*/}
        LOCK_FILE=/var/lock/subsys/${LOCK_FILE%.pid}
    fi

    local BASE=${1##*/}
    if [[ -n $PID_FILE ]] ; then
        BASE=${PID_FILE##*/} ; BASE=${BASE%.pid}
    fi

    # check /var/run/$PID_FILE.pid
    PID=$(get_pidfile -p "$PID_FILE" "$1")
    local RC=$?
    # check processes
    [ -z "$PID_FILE" -a -z "$PID" ] && PID=$(get_pidof $1)

    if [ -n "$PID" ]; then
        echo "$BASE (pid $PID) is running..."
        return 0
    fi

    case "$RC" in
        0)  echo "$BASE (pid $PID) is running..."
            return 0
            ;;
        1)  echo "$BASE dead but pid file exists"
            return 1
            ;;
        4)  echo "$BASE status unknown due to insufficient privileges."
            return 4
            ;;
    esac

    # see if /var/lock/subsys/$LOCK_FILE exists
    if [ -f $LOCK_FILE ]; then
        echo "$BASE dead but subsys locked"
        return 2
    fi
    echo "$BASE is stopped"
    return 3
}

# start runscript

export rs_start
rs_start() {
    # check syntax
    if [ "$#" = 0 ] ; then
        echo "Usage: rs_start [-bg] [-q|-Q] [-l logfile] [-u user] [-p pidfile] {program}"
        return 1
    fi

    local PID PID_FILE RUN_USER REDIR1 REDIR2 LOG_FILE BG
    unset PID PID_FILE RUN_USER REDIR1 REDIR2 LOG_FILE BG
    while [ $# -gt 0 ] ; do
        case $1 in
            -p) PID_FILE=$2 ; shift ; shift ;;
            -u) RUN_USER=$2 ; shift ; shift ;;
            -l) LOG_FILE=$2 ; shift ; shift ;;
            -q) REDIR1=' >/dev/null' ; shift ;;
            -Q) REDIR2=' 2>/dev/null' ; shift ;;
            -bg) BG=' &' ; shift ;;
             *) break ;;
        esac
    done

    if [[ ( -n ${!PID_FILE*} && -z $PID_FILE ) ||
          ( -n ${!RUN_USER*} && -z $RUN_USER ) ]] ; then
        echo "Usage: rs_start [-bg] [-q|-Q] [-l logfile] [-u user] [-p pidfile] {program}"
        return 1
    fi

    if [[ $CONSOLE != /dev/console ]] ; then
      local DAEMON=${PID_FILE:-$1}
      DAEMON=${DAEMON##*/}
      echo -n "Starting ${DAEMON%.pid} daemon: "
    fi

    if [[ -n $LOG_FILE ]] ; then
      mkdir -p $(dirname $LOG_FILE)
      if [[ -z "$REDIR1$REDIR2" || ( -n $REDIR1 && -n $REDIR2 ) ]] ; then
        REDIR1=" >>$LOG_FILE" ; REDIR2=' 2>&1'
      else
        [[ -n $REDIR1 ]] && REDIR1=" >>$LOG_FILE"
        [[ -n $REDIR2 ]] && REDIR2=" 2>>$LOG_FILE"
      fi
    fi

    # see if it's already running
    PID=$(get_pidfile -p "$PID_FILE" "$1")
    [ -n "$PID" ] && return

    # start it up
    if [[ -n $RUN_USER && $RUN_USER != root ]] ; then
        local SU_EXE=runuser
        $SU_EXE -c true 2>/dev/null || SU_EXE=su
        $SU_EXE $RUN_USER -c "$*$REDIR1$REDIR2$BG"
    else
        /bin/bash -c "$*$REDIR1$REDIR2$BG"
    fi
    rs_show $?

    # create pid file
    get_pidfile -p "$PID_FILE" "$1" >/dev/null
    return 0
}

# stop runscript

export rs_stop
rs_stop() {
    # check syntax
    if [ "$#" = 0 ]; then
        echo "Usage: rs_stop [-p pidfile] [-d delay] [program [-signal]]"
        return 1
    fi

    local PID PID_FILE DELAY
    unset PID PID_FILE DELAY
    while [ $# -gt 0 ] ; do
        case $1 in
            -p) PID_FILE=$2 ; shift ; shift ;;
            -d) DELAY=$2 ; shift ; shift ;;
             *) break ;;
        esac
    done

    if [[ ( -n ${!PID_FILE*} && -z $PID_FILE ) ||
          ( -n ${!DELAY*} && -z $DELAY ) ]] ; then
        echo "Usage: rs_stop [-p pidfile] [-d delay] [program [-signal]]"
        return 1
    fi
    DELAY=${DELAY//[^0-9]/}
    [[ -n $DELAY ]] || DELAY=3
    local SIGNAL=$2

    if [[ $CONSOLE != /dev/console ]] ; then
      local DAEMON=${PID_FILE:-$1}
      DAEMON=${DAEMON##*/}
      echo -n "Shutting down ${DAEMON%.pid} daemon: "
    fi

    # save basename
    local BASE=${1##*/}

    # check /var/run/$PID_FILE.pid
    PID=$(get_pidfile -p "$PID_FILE" "$1")
    local RC=$?
    # check processes
    if [ -z "$PID" ]; then
        if [ -z "$PID_FILE" ]; then
            PID=$(get_pidof $1)
        else
            [ "$RC" = "4" ] && { rs_show $RC; return $RC ;}
        fi
    fi

    if [ -z "$PID" ]; then
        # program is not running
        [ -z "$SIGNAL" ] && {
            rs_show 1; RC=3; rm -f "${PID_FILE:-/var/run/$BASE.pid}";
        }
        return $RC
    fi

    if [ -n "$SIGNAL" ] ; then
        # use specified level only
        if checkpid $PID; then
            kill $SIGNAL $PID >/dev/null 2>&1
            RC=$?
            rs_show $RC
        fi
        return $RC
    fi

    if checkpid $PID 2>&1; then
        # TERM first, then KILL if not dead
        kill -TERM $PID >/dev/null 2>&1
        usleep 100000
        WAITING=0
        while checkpid $PID ; do
          [[ $WAITING -ge $DELAY ]] && break
          sleep 1
          WAITING=$(($WAITING + 1))
        done
        kill -KILL $PID >/dev/null 2>&1
        usleep 100000
    fi
    checkpid $PID
    RC=$((! $?))
    rs_show $RC

    # remove pid file if any
    rm -f "${PID_FILE:-/var/run/$BASE.pid}"
    return $RC
}

# started_by_rpm

export started_by_rpm
started_by_rpm() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'started_by_rpm PID'
    echo "returns: 0 if one of ancestors of PID is rpm/yum, else 1"
    return 1
  fi

  ( get_pidof yum ; get_pidof zypp ; get_pidof zypper ) | grep -q '[0-9]' && return 0
  get_pidfile -p /var/run/yum.pid && return 0
  get_pidfile -p /var/run/zypp.pid && return 0

  USER=${LOGNAME:-$USER}
  ps xwww --forest | tac > /tmp/$USER-started_by_rpm.0
  sed -rn "/^\s+$1\s+/,\$p" /tmp/$USER-started_by_rpm.0 > /tmp/$USER-started_by_rpm.1
  echo PID=$1 >> /tmp/$USER-started_by_rpm.1
  while read -r pid tty stat time cmd ; do
    [[ "$cmd" != "${cmd#*started_by_rpm}" ]] && continue
    [[ "$cmd" != "${cmd#*rpm}" ]] && return 0
    [[ "$cmd" = "${cmd#*\_}" ]] && break
  done < /tmp/$USER-started_by_rpm.1
  return 1
}

# jre_home

export jre_home
jre_home() {
  if [[ $# -ne 0 ]] ; then
    echo 'jre_home'
    echo "returns: path to JRE directory"
    return 1
  fi
  local JRE EPAGES_JAVA

  EPAGES_JAVA=$(find /usr/java/latest -printf %l 2>/dev/null)
  if [[ $EPAGES_JAVA != ${EPAGES_JAVA#*jdk1.7} ]] ; then
    [ -d $EPAGES_JAVA/jre ] && { echo /usr/java/latest/jre; return; }
  fi

  case "$EPAGES_KERNEL" in
    *r) FLAVOUR=1.6.0-openjdk ;;
    *s) [[ -n $EPAGES_JAVA && -d $EPAGES_JAVA/jre ]] && { echo /usr/java/latest/jre; return; }
        FLAVOUR=1_6_0-ibm ;;
  esac

  for LIB in /usr/lib64 /usr/lib ; do
    EPAGES_JAVA=$(ls -d $LIB*/jvm/java-$FLAVOUR*/jre 2>/dev/null | sort | tail -1)
    [ -n "$EPAGES_JAVA" ] && { echo $EPAGES_JAVA; return; }
    [ -d "$JRE_HOME" ] && { echo $JRE_HOME; return; }

    EPAGES_JAVA=$(ls -d $LIB*/jvm/jre 2>/dev/null | sort | tail -1)
    [ -n "$EPAGES_JAVA" ] && { echo $EPAGES_JAVA; return; }
    EPAGES_JAVA=$(ls -d $LIB*/jvm/java-*/jre 2>/dev/null | sort | tail -1)
    [ -n "$EPAGES_JAVA" ] && { echo $EPAGES_JAVA; return; }
  done
}

# java_executable

export java_executable
java_executable() {
  if [[ $# -ne 0 ]] ; then
    echo 'java_executable'
    echo "returns: java executable path"
    return 1
  fi

  EPAGES_JAVA="$(jre_home)"
  [ -x "$EPAGES_JAVA"/bin/java ] && { echo "$EPAGES_JAVA"/bin/java; return; } 
  JAVA_HOME="$(dirname "$EPAGES_JAVA")"
  [ -x "$JAVA_HOME"/bin/java ] && { echo "$JAVA_HOME"/bin/java; return; } 
  type -p java
  return 0
}

# etc_default

export etc_default
etc_default() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'etc_default'
    echo "returns: path to epages default file (e.g. /etc/default/epages6)"
    return 1
  fi
  local PREFIX="$1"
  local PREFIXED=$PREFIX/etc/default/epages6

  [[ -n $PREFIX && -f $PREFIXED ]] && { echo $PREFIXED; return; }

  EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
  [[ -n $EPAGES_PROJECT ]] && { echo /etc/default/$EPAGES_PROJECT; return; }

  PREFIX=$(rpm -q epages-common --qf '[%{INSTPREFIXES}\n]' 2>/dev/null | head -1)
  PREFIXED=$PREFIX/etc/default/epages6
  [[ -n $PREFIX && -f $PREFIXED ]] && { echo $PREFIXED; return; }

  [[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6
  EPAGES_PROJECT=$EPAGES_INSTALL_RUN_SCRIPT
  [[ -n $EPAGES_PROJECT ]] && { echo /etc/default/$EPAGES_PROJECT; return; }

  [[ -f /etc/default/epages6 ]] && { echo /etc/default/epages6; return; }
  [[ $(ls /etc/default/epages6* 2>/dev/null | grep -v '~$' | wc -l) -eq 1 ]] \
  && { ls /etc/default/epages6* | grep -v '~$'; return; }
  echo /etc/default/epages6
}

# run_cron

export EPAGES_CRON_NO_SCRIPT=99
export EPAGES_CRON_IGNORED=98

export run_cron
run_cron() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'run_cron {start|stop|show} [USER]'
    echo "USER: starts/stops/shows epages cron processes (of USER)"
    return 1
  fi
  local RUN=$1
  local user=$2
  local sched="$EPAGES"/bin/epagesScheduler.sh
  local crond=/etc/init.d/cron
  [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%r} ]] && crond=${crond}d

  test -x "$sched" || return $EPAGES_CRON_NO_SCRIPT
  [[ -n $EPAGES_IGNORE_CRON ]] && return $EPAGES_CRON_IGNORED

  local TASK=" ${user:-epages} cron jobs..."
  case "$RUN" in
    start) TASK=$(Y -i Starting $TASK) ;;
    stop)  TASK=$(Y -i Stopping $TASK) ;;
    show)  TASK=$(Y -i Showing $TASK) ;;
    *) echo 'run_cron {start|stop|show} [USER]'
       echo "USER: starts/stops/shows epages cron processes (of USER)"
       return 1 ;;
  esac
  echo $TASK

  $crond status >/dev/null 2>&1 || { $crond status ; return $?; }

  if [[ -n $user ]] ; then
    local SU_EXE=runuser
    $SU_EXE -c true 2>/dev/null || SU_EXE=su
    $SU_EXE $user -c ". \"$(etc_default)\" ; LOGNAME=$user \"$sched\" $RUN" \
    2>/dev/null | grep -v '^S[a-z]*ing '
  else
   "$sched" $RUN | grep -v '^S[a-z]*ing '
  fi

  $crond status
  return $?
}

# mysqld_name

export mysqld_name
mysqld_name() {
  if [[ $# -ne 0 ]] ; then
    echo 'mysqld_name'
    echo "returns: name of MySQL daemon"
    return 1
  fi
  rpm -q --quiet epages-mysqld 2>/dev/null || return
  [[ -w /etc/init.d/mysql ]] && echo mysql || echo mysqld
}

# httpd_name

export httpd_name
httpd_name() {
  if [[ $# -ne 0 ]] ; then
    echo 'httpd_name'
    echo "returns: name of httpd daemon"
    return 1
  fi
  local HTTPD_RPM APACHE2_RPM

  rpm --quiet -q httpd && HTTPD_RPM=1
  rpm --quiet -q apache2 && APACHE2_RPM=1

  [[ -n $HTTPD_RPM && -f /etc/init.d/httpd ]] && { echo httpd; return; }
  [[ -n $APACHE2_RPM && -f /etc/init.d/apache2 ]] && { echo apache2; return; }

  [[ -f /etc/init.d/httpd ]] && { echo httpd; return; }
  [[ -f /etc/init.d/apache2 ]] && { echo apache2; return; }
}

# httpd_confd

export httpd_confd
httpd_confd() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'httpd_confd [epages]'
    echo "returns: path to httpd confdir (with epages configuration)"
    return 1
  fi
  local EPCONF
  [[ $1 = epages ]] && EPCONF=$1

  local NATIVE_HTTPD=$(httpd_name)
  [[ -z $NATIVE_HTTPD ]] && return
  [[ -d /etc/$NATIVE_HTTPD ]] || return

  for i in conf.d vhosts.d extra ; do
    if [[ -f "/etc/$NATIVE_HTTPD/$i/zzz-epages-httpd.conf" ]] ; then
      echo "/etc/$NATIVE_HTTPD/$i" ; return
    elif [[ -d "/etc/$NATIVE_HTTPD/$i" && $EPCONF != epages ]] ; then
      echo "/etc/$NATIVE_HTTPD/$i" ; return
    fi
  done
}

# check nfs client

export check_nfs
check_nfs() {
  if [[ $# -gt 0 ]] ; then
    echo 'check_nfs'
    echo "returns: 0 if NFS client process rpc.statd is running, else 1"
    return 1
  fi
  ps -Crpc.statd || { echo "rpc.statd: no such process"; return 1; }
  return 0
}

# httpd environment

unset EP_HTTPD EPAGES_REWRITE_CONF
if test -d "$EPAGES_APACHE" ; then
  if test -x "$EPAGES_APACHE"/bin/httpd.sh || [[ "$(httpd_confd)" = "" ]] ; then
    HTTPD_ROOT="$EPAGES_APACHE"
    EP_HTTPD="$HTTPD_ROOT/bin/apachectl"
    EPAGES_REWRITE_CONF="$HTTPD_ROOT/conf/extra/zzz-epages-rewrite.conf"
  fi
fi

if [[ -z $EPAGES_REWRITE_CONF ]] ; then
  NATIVE_HTTPD=$(httpd_name)
  CONF_D=$(httpd_confd)
  EPAGES_REWRITE_CONF=${CONF_D:-${HTTPD_ROOT:-/etc/${NATIVE_HTTPD:-httpd}}/conf.d}/zzz-epages-rewrite.conf
fi

export HTTPD_ROOT EP_HTTPD EPAGES_REWRITE_CONF

# perl environment

# perl socket class for https requests
# valid values: Net::SSL or IO::Socket::SSL
PERL_NET_HTTPS_SSL_SOCKET_CLASS=Net::SSL
# verify the hostname in the SSL certificate
# must be set to 0 if PERL_NET_HTTPS_SSL_SOCKET_CLASS=Net::SSL
# PERL_LWP_SSL_VERIFY_HOSTNAME= is same as PERL_LWP_SSL_VERIFY_HOSTNAME=0
PERL_LWP_SSL_VERIFY_HOSTNAME=0

export PERL_NET_HTTPS_SSL_SOCKET_CLASS PERL_LWP_SSL_VERIFY_HOSTNAME

# path

PATH=$(pathmunge "/bin:/usr/bin:/sbin:/usr/sbin" "$PATH")
PATH=$(pathmunge "$PATH:$EPAGES_PERL/bin:$HTTPD_ROOT/bin")
export PATH

export epages_path
epages_path() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'epages_path'
    echo "prepends PATH with EPAGES_PERL/bin"
    return 1
  fi
  pathmunge "$EPAGES_PERL/bin:$PATH"
}

# libraries

LD_LIBRARY_PATH=$(pathmunge "/lib:/usr/lib:/usr/X11R6/lib" "$LD_LIBRARY_PATH:$LD_LIBRARY_PATH_64")
export LD_LIBRARY_PATH

export epages_libs
epages_libs() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'epages_libs'
    echo "prepends LD_LIBRARY_PATH with epages libdirs"
    return 1
  fi
  pathmunge "$EPAGES_PERL/lib32/$EPAGES_KERNEL:$EPAGES_PERL/lib32/ssl:$EPAGES_PERL/bin:$HTTPD_ROOT/lib:$LD_LIBRARY_PATH"
}

# selinux

export set_selinux4epages
set_selinux4epages() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'set_selinux4epages'
    echo "sets SELinux file context for ePages files (if EPAGES_ENABLE_SELINUX is set"
    return 1
  fi
  [[ ${EPAGES_ENABLE_SELINUX:-0} != 0 ]] || return
  test -x $EPAGES/bin/se-file-context.sh || return
  [[ ${EPAGES_ENABLE_PERL_CGI:-0} = 0 ]] && PERL_CGI=--perl-cgi || unset PERL_CGI
  $EPAGES/bin/se-file-context.sh --set $PERL_CGI
}

# check_epages_webgroup

export check_epages_webgroup
check_epages_webgroup() {
  if [[ $# -lt 2 ]] ; then
    echo 'check_epages_web {*start*/*stop*} EP_DEFAULT'
    echo "EP_DEFAULT: checks (e.g. /etc/default/epages6) for EPAGES_WEBGROUP= setting"
    echo "*start*: add missing UNIX user as defined in EPAGES_WEBGROUP="
    echo "returns: GID of epages web group"
    return 1
  fi
  local EP_DEFAULT=$2
  local RESULT

  EPWEBID=${EPAGES_WEBGROUP#*:}
  EPWEB=${EPAGES_WEBGROUP%%:*}
  if [ -z "$EPWEB" ] ; then
    EPWEB=ep_web
    EPWEBID=`$AWK -F: "/^$EPWEB:/{ print \\$3; }" /etc/group`
  elif [ "$EPWEBID" != "${EPWEBID#*:}" ] ; then
    echo "$EP_DEFAULT: EPAGES_WEBGROUP=$EPAGES_WEBGROUP: wrong format of environment variable"
    exit 1
  fi

  case $1 in
  start|*start_httpd|start_ssl|start_service)
    $AWK -F: "/^$EPWEB:/{if (\$3 == \"$EPWEBID\") {exit 123} else exit 124}" /etc/group
    RESULT=$?
    if [ $RESULT = 124 ] ; then
      echo "$EP_DEFAULT: EPAGES_WEBGROUP=$EPAGES_WEBGROUP: void group name:id pair"
      exit 1
    elif [ $RESULT = 0 ] ; then
      groupadd -g $EPWEBID $EPWEB || {
        echo "$0: EPAGES_WEBGROUP=$EPAGES_WEBGROUP: command failed: groupadd -g $EPWEBID $EPWEB"
        exit 1
      }
    fi
    for cfg in group gshadow ; do
      [ -f /etc/$cfg ] || continue
      for user in ep_appl $EPAGES_APPUSER ; do
        egrep -q "^$EPWEB:.*[:,]$user(,|$)" /etc/$cfg || {
          sed -i "s/^\($EPWEB:.*[^:]\)$/\1,$user/;s/^\($EPWEB:.*:\)$/\1$user/" /etc/$cfg || {
            echo "$0: adding user '$user' to /etc/$cfg '$EPWEB' failed"
            exit 1
          }
        }
      done
    done
  ;;
  esac
  echo $EPWEBID
}

# my_host

export my_host
my_host() {
  if [[ -z $1 || $1 != ${1#-} ]] ; then
    echo 'my_host IP/hostname'
    echo "checks if IP/hostname belongs to host that executes my_host"
    return 1
  fi
  local ADDR=$(eval echo $1)

  MY_HOST=$(hostname --fqdn)
  [[ 0.0.0.0 = $ADDR || $MY_HOST = $ADDR || ${MY_HOST%%.*} = $ADDR ]] && return 0

  for IP in $(PATH=/sbin:/usr/sbin:$PATH ip addr show \
              | awk '/^[[:space:]]*inet/{sub(/\/.*/,"",$2); print $2}') ; do
    [[ $IP = $ADDR ]] && return 0
    getent ahosts $ADDR | grep -Pq "^$IP\s" && return 0
    dig +time=1 +short $ADDR +search +answer | grep -q "^$IP$" && return 0
  done
  return 1
}

# copy_ini_values

export copy_ini_values
copy_ini_values () {
  if [[ $# -lt 2 ]] || ! ls "$1" "$2" >/dev/null ; then
     echo 'copy_ini_values SOURCE_INI TARGET_INI'
     echo 'copies all values from SOURCE_INI to TARGET_INI'
     return 1
  fi
  grep -v '^[[:space:]]*[;#]' "$1" | sed 's,[;#][^\042\047]*$,,' > "$2".src || return 1

  PATH=/var/epages:$PATH
  for section in $(ini-config.sh -f "$2".src -g) ; do
    ini-config.sh -f "$2".src -s "$section" -g | while read line ; do
      key=$(echo ${line%%=*} | sed -r 's,(^ *| *$),,')
      value=$(echo ${line#*=} | sed -r 's,(^ *| *$),,')
      ini-config.sh -f "$2" -s "$section" -k "$key" -v "$value" -i
    done
  done
  rm -f "$2".src
}

#
# END
#
